#!/usr/bin/env python
# coding: utf-8

from argparse import (
    ArgumentParser,
    RawDescriptionHelpFormatter
)

description = """PYPI Classifiers for Humans

The pypicf asks you of PYPI Classifiers for setup.py script.
(https://pypi.org/pypi?%3Aaction=list_classifiers)
"""

parser = ArgumentParser(
    prog="pypicf",
    formatter_class=RawDescriptionHelpFormatter,
    description=description)

parser.add_argument(
    "--dev",
    action="store_true",
    help="Run the pypicf to development mode.")



#!/usr/bin/env python
# coding: utf-8

from inquirer import (
    Text,
    List,
    Password,
    Confirm,
    Checkbox,
    prompt
)

from .exceptions import EmptyError


class QuestionHandler:

    TEXT = "text"
    LIST = "list"
    PASSWORD = "password"
    CHECKBOX = "checkbox"
    CONFIRM = "confirm"


class InquirerComponent(object):

    #
    # Public
    # ##########################

    def prompt(self, handler, message, choices=None):

        if choices:
            questions = self.make_question(handler, message, choices=choices)

        else:
            questions = self.make_question(handler, message)

        result = prompt(questions, raise_keyboard_interrupt=True)[handler]

        if self._isempty(result):

            error_msg = "Empty result! "

            if (handler == QuestionHandler.TEXT
                    or handler == QuestionHandler.PASSWORD):
                error_msg += "Please write something."

            elif handler == QuestionHandler.LIST:
                error_msg += "Please select choiecs."

            elif handler == QuestionHandler.CHECKBOX:
                error_msg += "Please check checkbox."

            elif handler == QuestionHandler.CONFIRM:
                error_msg += "Please write Y/N."

            raise EmptyError(error_msg)

        else:
            return result

    def prompt_text(self, message):

        handler = QuestionHandler.TEXT
        result = self.prompt(handler, message)

        return result

    def prompt_password(self, message):

        handler = QuestionHandler.PASSWORD
        result = self.prompt(handler, message)

        return result

    def prompt_confirm(self, message):

        handler = QuestionHandler.CONFIRM
        result = self.prompt(handler, message)

        return result

    def prompt_list(self, message, choices):

        handler = QuestionHandler.LIST
        result = self.prompt(handler, message, choices)

        return result

    def prompt_checkbox(self, message, choices):

        handler = QuestionHandler.CHECKBOX
        result = self.prompt(handler, message, choices)

        return result

    def make_question(self, handler, message, **kwargs):

        if handler == QuestionHandler.TEXT:

            question = Text(handler, message=message)

        elif handler == QuestionHandler.PASSWORD:

            question = Password(handler, message=message)

        elif handler == QuestionHandler.CONFIRM:

            question = Confirm(handler, message=message)

        elif handler == QuestionHandler.LIST:

            if "choices" not in kwargs:
                raise KeyError("List question need 'choices' kwargs")

            choices = kwargs["choices"]

            description = "('↑', '↓':Select)"
            message = self._add_description(message, description)

            question = List(handler, message=message, choices=choices)

        elif handler == QuestionHandler.CHECKBOX:

            if "choices" not in kwargs:
                raise KeyError("Checkbox question need 'choices' kwargs")

            choices = kwargs["choices"]

            description = "('↑', '↓':Select, '→','←':Choose)"
            message = self._add_description(message, description)

            question = Checkbox(handler, message=message, choices=choices)

        return [question]

    #
    # Private
    # ##########################

    def _isempty(self, obj):

        if isinstance(obj, str):
            return True if obj == "" else False

        elif isinstance(obj, list):
            return True if len(obj) == 0 else False

    def _add_description(self, message, description):

        if message.endswith(" "):
            message += description

        else:
            message += " {0}".format(description)

        return message



#!/usr/bin/env python
# coding: utf-8

import huepy


class Logger(object):

    def pass_through(self, classifier_name):

        pass_through = huepy.bold(huepy.orange("passed through"))
        msg = "The {0} classifier has {1}.\n"
        print huepy.info(msg.format(classifier_name, pass_through))

    def require(self, classifier_name):

        required = huepy.bold(huepy.red("required"))
        msg = "The {0} classifier is {1}.\n"
        print huepy.bad(msg.format(classifier_name, required))

    def error(self, error_name, error_msg):

        error_name = huepy.bold(huepy.red(error_name))
        msg = "{0}: {1}\n"
        print huepy.bad(msg.format(error_name, error_msg))


logger = Logger()



#!/usr/bin/env python
# coding: utf-8

import functools

from .logging import logger
from .exceptions import SearchError


def search_database(query, database):

    result = []

    for data in database:

        foundflag = False
        testcase = [
            data,
            data.title(),
            data.capitalize(),
            data.upper(),
            data.lower(),
        ]

        for case in testcase:

            if foundflag:
                continue

            if case.find(query) != -1:
                foundflag = True
                result.append(data)

    if len(result) == 0:
        raise SearchError("Classifiers not found.")

    return result


def prettyprint(classifiers, indent=4):

    margin = " " * indent

    context = "classifiers=[\n"
    for classifier in classifiers:
        context += margin + "\"{0}\",\n".format(classifier)
    context += "]"

    print context


def require_classifier(classifier_name):
    def _decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            loop_flag = False
            while loop_flag is False:
                try:
                    return func(*args, **kwargs)
                    loop_flag = True

                except KeyboardInterrupt:
                    logger.require(classifier_name)
                    continue

                except Exception as e:
                    logger.error(type(e).__name__, e.message)
                    continue
        return wrapper
    return _decorator


def loop_function(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        loop_flag = False
        while loop_flag is False:
            try:
                return func(*args, **kwargs)
                loop_flag = True

            except Exception as e:
                logger.error(type(e).__name__, e.message)
                continue
    return wrapper



#!/usr/bin/env python
# coding: utf-8

from .component import InquirerComponent
from .logging import logger
from .classifiers import (
    # development status
    development_status_description,
    DevelopmentStatus,
    # environment
    environment_text_description,
    environment_list_description,
    Environment,
    # framework
    framework_text_description,
    framework_list_description,
    Framework,
    # intended audience
    intended_audience_description,
    IntendedAudience,
    # license
    license_text_description,
    license_list_description,
    License,
    # natural language
    natural_language_text_description,
    natural_language_list_description,
    NaturalLanguage,
    # operating system
    operating_system_description,
    OperatingSystem,
    # programming language
    programming_language_text_description,
    programming_language_list_description,
    ProgrammingLanguage,
    # topics
    topics_text_description,
    topics_list_description,
    Topics
)
from .utils import (
    search_database,
    prettyprint,
    require_classifier,
    loop_function
)


def program(args):

    # 0. Display banner

    print """________     ___    ___  ________    ___      ________      ________
|\   __  \   |\  \  /  /||\   __  \  |\  \    |\   ____\    |\  _____\\
\ \  \|\  \  \ \  \/  / /\ \  \|\  \ \ \  \   \ \  \___|    \ \  \__/
 \ \   ____\  \ \    / /  \ \   ____\ \ \  \   \ \  \        \ \   __\\
  \ \  \___|   \/  /  /    \ \  \___|  \ \  \   \ \  \____    \ \  \_|
   \ \__\    __/  / /       \ \__\      \ \__\   \ \_______\   \ \__\\
    \|__|   |\___/ /         \|__|       \|__|    \|_______|    \|__|
            \|___|/

Welcome to pypicf.
(1) Please select or check following classifiers.
(2) To select classifiers please use '↑', '↓' arrow keys.
(3) To check classifiers checkbox please use '→', '←' arrow keys.
(4) And you need to search before selecting the classifiers
    because the particular classifiers have many many choices.
(5) You can pass through the classifier by put <Ctrl-C> key at prompt.
    However, there are classifiers that you must be select absolutely.
"""

    inquirer = InquirerComponent()
    selected_classifiers = []

    # 1. Development Status

    @require_classifier("Development Status")
    def _development_status_prompt():
        result = inquirer.prompt_list(
            development_status_description, DevelopmentStatus)
        return result

    selected_classifiers.append(_development_status_prompt())

    # 2. Environment

    @loop_function
    def _environment_prompt():
        query = inquirer.prompt_text(environment_text_description)
        choices = search_database(query, Environment)
        result = inquirer.prompt_list(environment_list_description, choices)
        return result

    try:
        selected_classifiers.append(_environment_prompt())

    except KeyboardInterrupt:
        logger.pass_through("Environment")

    # 3. Framework

    @loop_function
    def _framework_prompt():
        query = inquirer.prompt_text(framework_text_description)
        choices = search_database(query, Framework)
        result = inquirer.prompt_checkbox(framework_list_description, choices)
        return result

    try:
        selected_classifiers += _framework_prompt()

    except KeyboardInterrupt:
        logger.pass_through("Environment")

    # 4. Intended Audience

    @require_classifier("Intended Audience")
    def _intended_audience_prompt():
        result = inquirer.prompt_list(
            intended_audience_description, IntendedAudience)
        return result

    selected_classifiers.append(_intended_audience_prompt())

    # 5. License

    @require_classifier("License")
    def _license_prompt():
        query = inquirer.prompt_text(license_text_description)
        choices = search_database(query, License)
        result = inquirer.prompt_list(license_list_description, choices)
        return result

    selected_classifiers.append(_license_prompt())

    # 6. Natural Langauge

    @loop_function
    def _natural_language_prompt():
        query = inquirer.prompt_text(natural_language_text_description)
        choices = search_database(query, NaturalLanguage)
        result = inquirer.prompt_list(
            natural_language_list_description, choices)
        return result

    try:
        selected_classifiers.append(_natural_language_prompt())

    except KeyboardInterrupt:
        logger.pass_through("Natural Language")

    # 7. Operating System

    @loop_function
    def _operating_system_prompt():
        result = inquirer.prompt_checkbox(
            operating_system_description, OperatingSystem)
        return result

    try:
        selected_classifiers += _operating_system_prompt()

    except KeyboardInterrupt:
        logger.pass_through("Operating System")

    # 8. Programming Language

    @require_classifier("Programming Language")
    def _programming_language_prompt():
        query = inquirer.prompt_text(programming_language_text_description)
        choices = search_database(query, ProgrammingLanguage)
        result = inquirer.prompt_checkbox(
            programming_language_list_description, choices)
        return result

    selected_classifiers += _programming_language_prompt()

    # 9. Topics

    @require_classifier("Topics")
    def _topics_prompt():
        query = inquirer.prompt_text(topics_text_description)
        choices = search_database(query, Topics)
        result = inquirer.prompt_checkbox(
            topics_list_description, choices)
        return result

    selected_classifiers += _topics_prompt()

    print "Thank you for answered!"
    print "Please copy following classifiers and paste it your setup.py.\n"
    print "-" * 50
    prettyprint(selected_classifiers)
    print "-" * 50
    return 0



#!/usr/bin/env python
# coding: utf-8

import sys
import traceback

from .cli import parser
from .core import program


def main(argv=sys.argv):

    args = parser.parse_args()

    try:
        return_code = program(args)
        sys.exit(return_code)

    except Exception as e:
        error_name = type(e).__name__
        stacktrace = traceback.format_exc()

        if args.dev:
            print stacktrace.strip()

        else:
            sys.stderr.write("{0}: {1}\n".format(error_name, e.message))


if __name__ == '__main__':

    main()



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, find_packages
from pypicf import __author__, __version__

description = "The pypicf is an interactive tool that asks some "
"questions about the PYPI Classifiers of your Pyt "
"hon product, and then generates classifiers list "
"for the insert to the setup.py script."

with open("requirements.txt", "r") as fp:
    requires = fp.read().strip().split("\n")

with open("README.rst", "r") as fp:
    long_description = fp.read()

setup(
    name="pypicf",
    author=__author__,
    description=description,
    long_description=long_description,
    version=__version__,
    license="MIT License",
    packages=find_packages(),
    install_requires=requires,
    entry_points={
        "console_scripts": [
            "pypicf=pypicf.__main__:main"
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.7",
        "Topic :: Utilities",
    ]
)
